@@ -1,12 +1,13 @@
 # -*- coding: utf-8 -*-
 
-__version__   = '2.2.4'
+__version__   = '2.2.5'
 __author__    = "Avinash Kak (kak@purdue.edu)"
-__date__      = '2023-March-24'                   
-__url__       = 'https://engineering.purdue.edu/kak/distDLS/DLStudio-2.2.4.html'
+__date__      = '2023-March-28'                   
+__url__       = 'https://engineering.purdue.edu/kak/distDLS/DLStudio-2.2.5.html'
 __copyright__ = "(C) 2023 Avinash Kak. Python Software Foundation."
 
 
+
 __doc__ = '''
 
 
@@ -19,69 +20,67 @@ __doc__ = '''
 
 DATA PREDICTION:
 
-    Let's say you have a sequence of observations recorded at regular
-    intervals.  These could, for example, be the price of a stock share
-    recorded every hour; the hourly recordings of electrical load at your
-    local power utility company; the mean average temperature recorded on an
-    annual basis; and so on.  We want to use the past observations to predict
-    the value of the next one.  Solving these types of problems is the focus
-    of the DataPrediction co-class of DLStudio.
+    Let's say you have a sequence of observations recorded at regular intervals.
+    These could, for example, be the price of a stock share recorded every hour;
+    the hourly recordings of electrical load at your local power utility company;
+    the mean average temperature recorded on an annual basis; and so on.  We want
+    to use the past observations to predict the value of the next one.  Solving
+    these types of problems is the focus of the DataPrediction co-class of
+    DLStudio.
 
     As a problem, data prediction has much in common with text analytics and
-    seq2seq processing, in the sense that the prediction at the next time
-    instant must be based on all previous observations in a manner similar to
-    what we do in text analytics where the next word is understood taking into
-    account all the previous words in a sentence.  However, there are three
-    significant differences between purely numerical data prediction problems
-    and text-based problems:
+    seq2seq processing, in the sense that the prediction at the next time instant
+    must be based on all previous observations in a manner similar to what we do
+    in text analytics where the next word is understood taking into account all
+    the previous words in a sentence.  However, there are three significant
+    differences between purely numerical data prediction problems and text-based
+    problems:
 
-    1) Data Normalization: As you know by this time, neural networks require
-       that your input data be normalized to the [0,1] interval, assuming it
-       consists of non-negative numbers, or the [-1,1] interval otherwise.
-       This is dictated by the design of the data processing components made
-       available by the commonly used platforms.  For example, a commonly used
-       activation function, Sigmoid, saturates out outside the [0,1] interval;
-       and the activation function Tanh saturates out outside the [-1,1]
-       interval. For multi-channel inputs, with each channel being represented
-       by a different axis of the input tensor, such normalization is
-       generally carried out separately for each channel.
+    1) Data Normalization: As you know by this time, neural networks require that
+       your input data be normalized to the [0,1] interval, assuming it consists
+       of non-negative numbers, or the [-1,1] interval otherwise.  This is
+       dictated by the design of the data processing components made available by
+       the commonly used platforms.  For example, a commonly used activation
+       function, Sigmoid, saturates out outside the [0,1] interval; and the
+       activation function Tanh saturates out outside the [-1,1] interval. For
+       multi-channel inputs, with each channel being represented by a different
+       axis of the input tensor, such normalization is generally carried out
+       separately for each channel.
 
-       When solving a problem like text analytics, after you have normalized
-       the input data (which is likely to consist of the numeric embeddings
-       for the input words), you can forget about it.  You don't have that
-       luxury when solving a data prediction problem.  As you would expect,
-       the next value predicted by an algorithm must be at the same scale as
-       the original input input data.  This requires that the output of a
-       neural-network-based prediction algorithm must be "inverse-normalized".
-       And that, in turn, requires remembering the normalization parameters
-       used in each channel of the input data.
+       When solving a problem like text analytics, after you have normalized the
+       input data (which is likely to consist of the numeric embeddings for the
+       input words), you can forget about it.  You don't have that luxury when
+       solving a data prediction problem.  As you would expect, the next value
+       predicted by an algorithm must be at the same scale as the original input
+       input data.  This requires that the output of a neural-network-based
+       prediction algorithm must be "inverse-normalized".  And that, in turn,
+       requires remembering the normalization parameters used in each channel of
+       the input data.
 
     2) Input Data Chunking: The notion of a sentence that is important in text
-       analytics does not carry over to the data prediction problem.  In
-       general, you would want a prediction to be made using ALL of the past
-       observations.  You are also likely to believe that the longer the time
-       span over which the past observations are available, the greater the
-       likelihood that the prediction made for the next time instant will be
-       accurate.  When the sequential data available for training a predictor
-       is arbitrarily long, as is the case with numerical data in general, you
-       would need to decide how to "chunk" the data --- that is, how to
-       extract sub-sequences from the data for the purpose of training a
-       neural network.
+       analytics does not carry over to the data prediction problem.  In general,
+       you would want a prediction to be made using ALL of the past observations.
+       You are also likely to believe that the longer the time span over which the
+       past observations are available, the greater the likelihood that the
+       prediction made for the next time instant will be accurate.  When the
+       sequential data available for training a predictor is arbitrarily long, as
+       is the case with numerical data in general, you would need to decide how to
+       "chunk" the data --- that is, how to extract sub-sequences from the data
+       for the purpose of training a neural network.
 
-    3) Data Conditioning: Data prediction problems give you data as a time
-       series --- meaning data values recorded at regular intervals.  A
-       datafile could be as rudimentary as consisting of two comma separated
-       columns, one containing the "datetime" of the observation and other the
-       actual value for what's being observed.  To illustrate what I am
-       saying, let's consider the entries in one of the data files for the
-       utility power load prediction problem at:
+    3) Data Conditioning: Data prediction problems give you data as a time series
+       --- meaning data values recorded at regular intervals.  A datafile could be
+       as rudimentary as consisting of two comma separated columns, one containing
+       the "datetime" of the observation and other the actual value for what's
+       being observed.  To illustrate what I am saying, let's consider the entries
+       in one of the data files for the utility power load prediction problem at:
 
             https://www.kaggle.com/robikscube/hourly-energy-consumption    
 
        The dataset consists of over 10-years worth of hourly electric load
-       recordings made available by several utilities in the east and the
-       midwest of the United States.  Shown below is the start of a datafile
-       typical datafile:
+       recordings made available by several utilities in the east and the midwest
+       of the United States.  Shown below is the start of a datafile typical
+       datafile:
 
              2013-12-31 01:00:00,  1861.0
              2013-12-31 02:00:00,  1835.0
@@ -95,19 +94,19 @@ DATA PREDICTION:
 
        As you can see, the datafile has two comma separated columns: the left
        column is the datetime and the right the electrical load recorded in
-       megawatts at that time.  As far as the data is concerned, obviously
-       there's not much to be done with the actual load observations in the
-       right column since those values are purely one-dimensional.
+       megawatts at that time.  As far as the data is concerned, obviously there's
+       not much to be done with the actual load observations in the right column
+       since those values are purely one-dimensional.
 
-       However, it is an entirely different matter with regard to the datetime
-       in the left column. You see, for the case of predicting the electric
-       power load at utility company, the time of the day matters, as does the
-       day of the week, what season it is (day of the year), etc.  What that
-       means that it would make no sense to encode the datetime of the
-       observations as a one-dimensional entity. You would need to encode it
-       so that it has multiple degrees of freedom in a learning framework.  In
-       a GRU-based solution for the electric load prediction problem by
-       Gabriel Loye, the datetime is encoded into four numbers as follows:
+       However, it is an entirely different matter with regard to the datetime in
+       the left column. You see, for the case of predicting the electric power
+       load at utility company, the time of the day matters, as does the day of
+       the week, what season it is (day of the year), etc.  What that means that
+       it would make no sense to encode the datetime of the observations as a
+       one-dimensional entity. You would need to encode it so that it has multiple
+       degrees of freedom in a learning framework.  In a GRU-based solution for
+       the electric load prediction problem by Gabriel Loye, the datetime is
+       encoded into four numbers as follows:
 
          -- Hour of the day with an integer value between 0 and 23
          -- Day of the week as an integer value between 1 and 7
@@ -116,60 +115,57 @@ DATA PREDICTION:
 
        Data conditioning here would refer to extracting this four dimensional
        information from the datetime column shown above.  As it turns out, the
-       Pandas module comes in handy for that, as demonstrated by Gabriel Loye
-       in his code that you can download from:
+       Pandas module comes in handy for that, as demonstrated by Gabriel Loye in
+       his code that you can download from:
 
             https://blog.floydhub.com/gru-with-pytorch/
 
                             ---------------------
 
-    Now that you understand how the data prediction problem differs from, say,
-    the problem of text analytics, it is time for me to state my main goal in
-    defining the DataPrediction class in this file.  I actually have two
-    goals:
+    Now that you understand how the data prediction problem differs from, say, the
+    problem of text analytics, it is time for me to state my main goal in defining
+    the DataPrediction class in this file.  I actually have two goals:
 
-    (a) To deepen your understanding of a GRU.  At this point, your
-        understanding of a GRU is likely to be based on calling PyTorch's GRU
-        in your own code.  Using a pre-programmed implementation for a GRU
-        makes life easy and you also get a piece of highly optimized code that
-        you can just call in your own code.  However, with a pre-programmed
-        GRU, you are unlikely to get insights into how such an RNN is actually
-        implemented.
+    (a) To deepen your understanding of a GRU.  At this point, your understanding
+        of a GRU is likely to be based on calling PyTorch's GRU in your own code.
+        Using a pre-programmed implementation for a GRU makes life easy and you
+        also get a piece of highly optimized code that you can just call in your
+        own code.  However, with a pre-programmed GRU, you are unlikely to get
+        insights into how such an RNN is actually implemented.
 
-    (b) To demonstrate how you can use a Recurrent Neural Network (RNN) for
-        data prediction taking into account the data normalization, chunking,
-        and conditioning issues mentioned earlier.
+    (b) To demonstrate how you can use a Recurrent Neural Network (RNN) for data
+        prediction taking into account the data normalization, chunking, and
+        conditioning issues mentioned earlier.
 
-    To address the first goal above, the DataPrediction class presented in
-    this file is based on my pmGRU (Poor Man's GRU).  This GRU is my
-    implementation of the "Minimal Gated Unit" GRU variant that was first
-    presented by Joel Heck and Fathi Salem in their paper "Simplified Minimal
-    Gated Unit Variations for Recurrent Neural Networks".  Its hallmark is
-    that it combines the Update and the Reset gates of a regular GRU into a
-    single gate called the Forget Gate.  You could say that pmGRU is a
-    lightweight version of a regular GRU and its use may therefore lead to a
-    slight loss of accuracy in the predictions.  As to whether that loss would
-    be acceptable would depend obviously on the application context and the
-    needs of the customer.  You will find it educational to compare the
-    performance you get with my pmGRU-based implementation with Gabriel Loye's
-    implementation that uses PyTorch's GRU for the same dataset.  And after
-    you have seen my implementation of pmGRU, you should yourself be able to
-    translate the regular GRU gating equations into your own implementation
-    --- just as a learning exercise.
+    To address the first goal above, the DataPrediction class presented in this
+    file is based on my pmGRU (Poor Man's GRU).  This GRU is my implementation of
+    the "Minimal Gated Unit" GRU variant that was first presented by Joel Heck and
+    Fathi Salem in their paper "Simplified Minimal Gated Unit Variations for
+    Recurrent Neural Networks".  Its hallmark is that it combines the Update and
+    the Reset gates of a regular GRU into a single gate called the Forget Gate.
+    You could say that pmGRU is a lightweight version of a regular GRU and its use
+    may therefore lead to a slight loss of accuracy in the predictions.  As to
+    whether that loss would be acceptable would depend obviously on the
+    application context and the needs of the customer.  You will find it
+    educational to compare the performance you get with my pmGRU-based
+    implementation with Gabriel Loye's implementation that uses PyTorch's GRU for
+    the same dataset.  And after you have seen my implementation of pmGRU, you
+    should yourself be able to translate the regular GRU gating equations into
+    your own implementation --- just as a learning exercise.
 
-    For measuring the accuracy of his data prediction networks, Gabriel Loye
-    used the Symmetric Mean Absolute Percentage Error (sMAPE) that is the sum
-    of the absolute difference between the predicted and the actual values
-    divided by the average of the predicted and the actual values:
+    For measuring the accuracy of his data prediction networks, Gabriel Loye used
+    the Symmetric Mean Absolute Percentage Error (sMAPE) that is the sum of the
+    absolute difference between the predicted and the actual values divided by the
+    average of the predicted and the actual values:
 
                            100                     | P_t   -   GT_t |
             sMAPE    =    -----  \SUM_{t=1}^N  ---------------------------
                             N                    | P_t   +   GT_t | / 2.0
 
     where N is the number of predictions made in a file.  P_t is the value
-    predicted by the network at inference time and GT_t is the ground-truth
-    value that actually occurred at that time.  I have used the same metric
-    for estimating the accuracy of the prediction results produced by pmGRU.
+    predicted by the network at inference time and GT_t is the ground-truth value
+    that actually occurred at that time.  I have used the same metric for
+    estimating the accuracy of the prediction results produced by pmGRU.
 
 @endofdocs
 '''
