@@ -46,9 +46,10 @@ os.environ['PYTHONHASHSEED'] = str(seed)
 from DLStudio import *
 from Transformers import *
 
-dataroot = "/home/kak/TextDatasets/en_es_corpus_xformer/"
+dataroot = "./data/"
+#dataroot = "/home/kak/TextDatasets/en_es_corpus_xformer/"
 #dataroot = "/mnt/cloudNAS3/Avi/TextDatasets/en_es_corpus_xformer/"
-#dataroot = "./data/"
+
 
 data_archive =  "en_es_xformer_8_90000.tar.gz"
 
