@@ -1,12 +1,13 @@
 # -*- coding: utf-8 -*-
 
-__version__   = '2.2.4'
+__version__   = '2.2.5'
 __author__    = "Avinash Kak (kak@purdue.edu)"
-__date__      = '2023-March-24'                   
-__url__       = 'https://engineering.purdue.edu/kak/distDLS/DLStudio-2.2.4.html'
+__date__      = '2023-March-28'                   
+__url__       = 'https://engineering.purdue.edu/kak/distDLS/DLStudio-2.2.5.html'
 __copyright__ = "(C) 2023 Avinash Kak. Python Software Foundation."
 
 
+
 __doc__ = '''
 
 
@@ -19,77 +20,74 @@ __doc__ = '''
 
 SEQUENCE-TO-SEQUENCE LEARNING WITH ATTENTION: 
 
-    As mentioned in the main doc page for DLStduio, sequence-to-sequence
-    learning (seq2seq) is about predicting an outcome sequence from a
-    causation sequence, or, said another way, a target sequence from a source
-    sequence.  Automatic machine translation is probably one of the most
-    popular applications of seq2seq.  DLStudio uses English-to-Spanish
-    translation to illustrate the programming idioms and the PyTorch
-    structures you need for seq2seq.  
+    As mentioned in the main doc page for DLStduio, sequence-to-sequence learning
+    (seq2seq) is about predicting an outcome sequence from a causation sequence,
+    or, said another way, a target sequence from a source sequence.  Automatic
+    machine translation is probably one of the most popular applications of
+    seq2seq.  DLStudio uses English-to-Spanish translation to illustrate the
+    programming idioms and the PyTorch structures you need for seq2seq.
 
-    In version 2.0.9, where I first introduced seq2seq in DLStudio, I placed
-    the code inside the main DLStudio class.  However, considering that
-    seq2seq is an active area of research and development unto itself, it
-    seems more prudent to give seq2seq its own home.  Therefore, I have moved
-    that code to a co-class of DLStudio --- the class you are looking at as
-    you are reading this doc page.  This class, Seq2SeqLearning, resides at
-    the top-level of the distro.  That makes Seq2SeqLearning a co-class of the
-    main DLStudio class.
+    In version 2.0.9, where I first introduced seq2seq in DLStudio, I placed the
+    code inside the main DLStudio class.  However, considering that seq2seq is an
+    active area of research and development unto itself, it seems more prudent to
+    give seq2seq its own home.  Therefore, I have moved that code to a co-class of
+    DLStudio --- the class you are looking at as you are reading this doc page.
+    This class, Seq2SeqLearning, resides at the top-level of the distro.  That
+    makes Seq2SeqLearning a co-class of the main DLStudio class.
 
-    In Version 2.1.0, Seq2SeqLearning contains the following two inner
-    classes for illustrating seq2seq:
+    In Version 2.1.0, Seq2SeqLearning contains the following two inner classes for
+    illustrating seq2seq:
     
         1.  Seq2SeqWithLearnableEmbeddings
 
         2.  Seq2SeqWithPretrainedEmbeddings
 
-    As their names imply, the first is for seq2seq with learnable
-    embeddings and the second for seq2seq with pre-trained embeddings like
-    word2vec or fasttext.  The seq2seq implementations include the
-    attention mechanism based on my understanding of the original paper on
-    the subject by Bahdanau, Cho, and Bengio.
+    As their names imply, the first is for seq2seq with learnable embeddings and
+    the second for seq2seq with pre-trained embeddings like word2vec or fasttext.
+    The seq2seq implementations include the attention mechanism based on my
+    understanding of the original paper on the subject by Bahdanau, Cho, and
+    Bengio.
 
     As mentioned in the first para above, the specific example of seq2seq
-    addressed in my implementation code is translation from English to
-    Spanish. (I chose this example because learning and keeping up with
-    Spanish is one of my hobbies.)  In the Seq2SeqWithLearnableEmbeddings
-    class, the learning framework learns the best embedding vectors to use
-    for the two languages involved. On the other hand, in the
-    Seq2SeqWithPretrainedEmbeddings class, I use the word2vec embeddings
-    provided by Google for the source language.  As to why I use the
-    pre-training embeddings for just the source language is explained in the
-    main comment doc associated with the class
+    addressed in my implementation code is translation from English to Spanish. (I
+    chose this example because learning and keeping up with Spanish is one of my
+    hobbies.)  In the Seq2SeqWithLearnableEmbeddings class, the learning framework
+    learns the best embedding vectors to use for the two languages involved. On
+    the other hand, in the Seq2SeqWithPretrainedEmbeddings class, I use the
+    word2vec embeddings provided by Google for the source language.  As to why I
+    use the pre-training embeddings for just the source language is explained in
+    the main comment doc associated with the class
     Seq2SeqWithPretrainedEmbeddings.
 
-    To repeat what I mentioned on the main DLStudio doc page, any modern
-    attempt at seq2seq must include attention.  This is done by incorporating
-    a separate Attention network in the Encoder-Decoder framework needed for
-    seq2seq learning.  The goal of the attention network is to modify the
-    current hidden state in the decoder using the attention units produced
-    previously by the encoder for the source language sentence.  
+    To repeat what I mentioned on the main DLStudio doc page, any modern attempt
+    at seq2seq must include attention.  This is done by incorporating a separate
+    Attention network in the Encoder-Decoder framework needed for seq2seq
+    learning.  The goal of the attention network is to modify the current hidden
+    state in the decoder using the attention units produced previously by the
+    encoder for the source language sentence.
 
-    As mentioned above, the main Attention model I have used is based on my 
-    understanding of the attention mechanism proposed by Bahdanau, Cho, and 
-    Bengio. You will see this attention code in a class named Attention_BCB 
-    of Seq2SeqWithLearnableEmbeddings.  I have also provided another 
-    attention class named Attention_SR that is my implementation of the 
-    attention mechanism in the very popular NLP tutorial by Sean Robertson 
-    at the PyTorch website.  The URLs to both these attention mechanisms 
-    are in my Week 14 lecture material on deep learning at Purdue.
+    As mentioned above, the main Attention model I have used is based on my
+    understanding of the attention mechanism proposed by Bahdanau, Cho, and
+    Bengio. You will see this attention code in a class named Attention_BCB of
+    Seq2SeqWithLearnableEmbeddings.  I have also provided another attention class
+    named Attention_SR that is my implementation of the attention mechanism in the
+    very popular NLP tutorial by Sean Robertson at the PyTorch website.  The URLs
+    to both these attention mechanisms are in my Week 14 lecture material on deep
+    learning at Purdue.
 
     The following two scripts in the ExamplesSeq2SeqLearning directory of the
-    distribution are your main entry points for experimenting with the
-    seq2seq code:
+    distribution are your main entry points for experimenting with the seq2seq
+    code:
 
         1.  seq2seq_with_learnable_embeddings.py
 
         2.  seq2seq_with_pretrained_embeddings.py
     
     With the first script, the overall network will learn on its own the best
-    embeddings to use for representing the words in the two languages.  And,
-    with the second script, the pre-trained word2vec embeddings from Google
-    are used for the source language while the system learns the embeddings
-    for the target language.
+    embeddings to use for representing the words in the two languages.  And, with
+    the second script, the pre-trained word2vec embeddings from Google are used
+    for the source language while the system learns the embeddings for the target
+    language.
 
 @endofdocs
 '''
